--- ./doc/geany.txt.orig	2011-07-30 14:02:32.000000000 +0300
+++ ./doc/geany.txt	2011-08-16 20:38:39.000000000 +0300
@@ -4057,18 +4057,20 @@
 Values that are set in the [build-menu] section will override those in this section.
 
 error_regex
-    This is a GNU-style extended regular expression to parse a filename
-    and line number from build output. If undefined, Geany will fall
+    This is a GNU-style extended regular expression to parse a filename, a line
+    number and an optional column from build output. If undefined, Geany will fall
     back to its default error message parsing.
 
-    Only the first two matches will be read by Geany. Geany will look for
-    a match that is purely digits, and use this for the line number. The
-    remaining match will be used as the filename.
+    Geany will read the first two (or three, if there is a third) matches, and check
+    whether the first or second match is purely digits. If so, it'll try to use the
+    matches as <line> [column] <filename>, or as <filename> <line> [column].
 
-    *Example:* ``error_regex=(.+):([0-9]+):[0-9]+``
+    *Example:* ``error_regex=([^:]+):([0-9]+):([0-9]+): |([^:]+):([0-9]+): ``
 
     This will parse a message such as:
     ``test.py:7:24: E202 whitespace before ']'``
+    or (if the first part does not match) as:
+    ``test.py:7 E202 whitespace before ']'``
 
 **Build commands**
 
--- ./src/filetypes.c.orig	2011-08-14 14:52:08.000000000 +0300
+++ ./src/filetypes.c	2011-08-18 20:01:20.000000000 +0300
@@ -1510,13 +1510,14 @@
 
 
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
-		gchar **filename, gint *line)
+		gchar **filename, gint *line, gint *col)
 {
 	gchar *regstr;
 	gchar **tmp;
 	GeanyDocument *doc;
 	regex_t *regex;
-	regmatch_t pmatch[3];
+	regmatch_t pmatch[4];
+	gint has_column;
 
 	if (ft == NULL)
 	{
@@ -1531,7 +1532,7 @@
 	regex = &ft->priv->error_regex;
 
 	*filename = NULL;
-	*line = -1;
+	*line = *col = -1;
 
 	if (G_UNLIKELY(! NZV(regstr)))
 		return FALSE;
@@ -1547,35 +1548,48 @@
 	if (regexec(regex, message, G_N_ELEMENTS(pmatch), pmatch, 0) != 0)
 		return FALSE;
 
+	if (pmatch[2].rm_so == -1 && pmatch[3].rm_so != -1)
+	{
+		pmatch[2].rm_so = pmatch[3].rm_so;
+		pmatch[2].rm_eo = pmatch[3].rm_eo;
+		has_column = 0;
+	}
+	else
+	{
+		has_column = pmatch[3].rm_so != -1;
+	}
+
 	if (pmatch[0].rm_so != -1 && pmatch[1].rm_so != -1 && pmatch[2].rm_so != -1)
 	{
-		gchar *first, *second, *end;
-		glong l;
+		gchar *match[3];
+		int line_idx;
 
-		first = get_regex_match_string(message, pmatch, 1);
-		second = get_regex_match_string(message, pmatch, 2);
-		l = strtol(first, &end, 10);
-		if (*end == '\0')	/* first is purely decimals */
+		match[0] = get_regex_match_string(message, pmatch, 1);
+		match[1] = get_regex_match_string(message, pmatch, 2);
+		match[2] = has_column ? get_regex_match_string(message, pmatch, 3) : NULL;
+
+		if (utils_str_to_pos(match[0], line, TRUE) != NULL)	/* first is purely decimals */
 		{
-			*line = l;
-			g_free(first);
-			*filename = second;
+			line_idx = 0;
+			*filename = match[1 + has_column];
 		}
 		else
 		{
-			l = strtol(second, &end, 10);
-			if (*end == '\0')
-			{
-				*line = l;
-				g_free(second);
-				*filename = first;
-			}
+			line_idx = 1;
+			if (utils_str_to_pos(match[1], line, TRUE) != NULL)
+				*filename = match[0];
 			else
-			{
-				g_free(first);
-				g_free(second);
-			}
+				g_free(match[0]);
 		}
+
+		if (has_column)
+		{
+			if (*filename != NULL)
+				utils_str_to_pos(match[line_idx + 1], col, TRUE);
+			g_free(match[line_idx + 1]);
+		}
+
+		g_free(match[line_idx]);
 	}
 	return *filename != NULL;
 }
--- ./src/filetypes.h.orig	2011-05-20 20:07:51.000000000 +0300
+++ ./src/filetypes.h	2011-08-15 19:38:35.000000000 +0300
@@ -205,6 +205,6 @@
 gboolean filetype_has_tags(GeanyFiletype *ft);
 
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
-		gchar **filename, gint *line);
+		gchar **filename, gint *line, gint *col);
 
 #endif
--- ./src/highlighting.c.orig	2011-06-29 21:06:09.000000000 +0300
+++ ./src/highlighting.c	2011-08-18 19:52:31.000000000 +0300
@@ -332,16 +332,6 @@
 }
 
 
-static void convert_int(const gchar *int_str, gint *val)
-{
-	gchar *end;
-	gint v = strtol(int_str, &end, 10);
-
-	if (int_str != end)
-		*val = v;
-}
-
-
 /* Get first and second integer numbers, store in foreground and background fields of @a style. */
 static void get_keyfile_int(GKeyFile *config, GKeyFile *configh, const gchar *section,
 							const gchar *key, gint fdefault_val, gint sdefault_val,
@@ -366,10 +356,10 @@
 
 	if (list[0])
 	{
-		convert_int(list[0], &style->foreground);
+		utils_str_to_pos(list[0], &style->foreground, TRUE);
 		if (list[1])
 		{
-			convert_int(list[1], &style->background);
+			utils_str_to_pos(list[1], &style->background, TRUE);
 		}
 	}
 	g_strfreev(list);
--- ./src/msgwindow.c.orig	2011-08-14 14:52:08.000000000 +0300
+++ ./src/msgwindow.c	2011-08-22 21:06:59.000000000 +0300
@@ -60,6 +60,7 @@
 	const gchar *pattern;	/* pattern to split the error message into some fields */
 	guint min_fields;		/* used to detect errors after parsing */
 	guint line_idx;			/* idx of the field where the line is */
+	guint col_idx;			/* idx of the field where the column is or line_idx or -1 */
 	gint file_idx;			/* idx of the field where the filename is or -1 */
 }
 ParseData;
@@ -627,7 +628,8 @@
 }
 
 
-static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboolean focus_editor)
+static gboolean goto_compiler_file_line(const gchar *filename, gint line, gint col,
+		gboolean focus_editor)
 {
 	if (!filename || line <= -1)
 		return FALSE;
@@ -676,7 +678,7 @@
 			if (! doc->changed && editor_prefs.use_indicators)	/* if modified, line may be wrong */
 				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
 
-			ret = navqueue_goto_line(old_doc, doc, line);
+			ret = navqueue_goto_line_col(old_doc, doc, line, col);
 			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 
@@ -711,7 +713,7 @@
 		gtk_tree_model_get(model, &iter, 1, &string, -1);
 		if (string != NULL)
 		{
-			gint line;
+			gint line, col;
 			gchar *filename, *dir;
 			GtkTreePath *path;
 			gboolean ret;
@@ -719,11 +721,11 @@
 			path = gtk_tree_model_get_path(model, &iter);
 			find_prev_build_dir(path, model, &dir);
 			gtk_tree_path_free(path);
-			msgwin_parse_compiler_error_line(string, dir, &filename, &line);
+			msgwin_parse_compiler_error_line(string, dir, &filename, &line, &col);
 			g_free(string);
 			g_free(dir);
 
-			ret = goto_compiler_file_line(filename, line, focus_editor);
+			ret = goto_compiler_file_line(filename, line, col, focus_editor);
 			g_free(filename);
 			return ret;
 		}
@@ -756,13 +758,13 @@
  * relevant file with the error in *filename.
  * *line will be -1 if no error was found in string.
  * *filename must be freed unless it is NULL. */
-static void parse_file_line(ParseData *data, gchar **filename, gint *line)
+static void parse_file_line(ParseData *data, gchar **filename, gint *line, gint *col)
 {
-	gchar *end = NULL;
+	const gchar *end;
 	gchar **fields;
 
 	*filename = NULL;
-	*line = -1;
+	*line = *col = -1;
 
 	g_return_if_fail(data->string != NULL);
 
@@ -775,15 +777,24 @@
 		return;
 	}
 
-	*line = strtol(fields[data->line_idx], &end, 10);
+	end = utils_str_to_pos(fields[data->line_idx], line, FALSE);
 
-	/* if the line could not be read, line is 0 and an error occurred, so we leave */
-	if (fields[data->line_idx] == end)
+	/* if the line could not be read or is invalid, leave */
+	if (end == NULL)
 	{
 		g_strfreev(fields);
 		return;
 	}
 
+	/* parse the column */
+	if (data->col_idx == data->line_idx)
+	{
+		while (*end && !g_ascii_isdigit(*end)) end++;
+		utils_str_to_pos(end, col, FALSE);
+	}
+	else if (data->col_idx != -1 && g_strv_length(fields) >= data->col_idx)
+		utils_str_to_pos(fields[data->col_idx], col, FALSE);
+
 	/* let's stop here if there is no filename in the error message */
 	if (data->file_idx == -1)
 	{
@@ -801,9 +812,9 @@
 
 
 static void parse_compiler_error_line(const gchar *string,
-		gchar **filename, gint *line)
+		gchar **filename, gint *line, gint *col)
 {
-	ParseData data = {NULL, NULL, 0, 0, 0};
+	ParseData data = {NULL, NULL, 0, 0, -1, 0};
 
 	data.string = string;
 
@@ -862,6 +873,7 @@
 			data.pattern = "(";
 			data.min_fields = 2;
 			data.line_idx = 1;
+			data.col_idx = 1;
 			data.file_idx = 0;
 			break;
 		}
@@ -915,6 +927,7 @@
 			data.pattern = " ";
 			data.min_fields = 4;
 			data.line_idx = 1;
+			data.col_idx = 3;
 			data.file_idx = -1;
 			break;
 		}
@@ -952,6 +965,7 @@
 				data.pattern = ":";
 				data.min_fields = 3;
 				data.line_idx = 1;
+				data.col_idx = 2;
 				data.file_idx = 0;
 				break;
 			}
@@ -959,7 +973,7 @@
 	}
 
 	if (data.pattern != NULL)
-		parse_file_line(&data, filename, line);
+		parse_file_line(&data, filename, line, col);
 }
 
 
@@ -969,13 +983,13 @@
  * *line will be -1 if no error was found in string.
  * *filename must be freed unless it is NULL. */
 void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
-		gchar **filename, gint *line)
+		gchar **filename, gint *line, gint *col)
 {
 	GeanyFiletype *ft;
 	gchar *trimmed_string;
 
 	*filename = NULL;
-	*line = -1;
+	*line = *col = -1;
 
 	if (G_UNLIKELY(string == NULL))
 		return;
@@ -990,10 +1004,10 @@
 	ft = filetypes[build_info.file_type_id];
 
 	/* try parsing with a custom regex */
-	if (!filetypes_parse_error_message(ft, trimmed_string, filename, line))
+	if (!filetypes_parse_error_message(ft, trimmed_string, filename, line, col))
 	{
 		/* fallback to default old-style parsing */
-		parse_compiler_error_line(trimmed_string, filename, line);
+		parse_compiler_error_line(trimmed_string, filename, line, col);
 	}
 	make_absolute(filename, dir);
 	g_free(trimmed_string);
@@ -1022,10 +1036,9 @@
 		/* now the line */
 		if (fields[1] != NULL)
 		{
-			gchar *end;
+			const gchar *end = utils_str_to_pos(fields[1], line, FALSE);
 
-			*line = strtol(fields[1], &end, 10);
-			if (end == fields[1])
+			if (end == NULL)
 				*line = -1;
 			else if (*end == ':' || g_ascii_isspace(*end))
 			{	/* if we have a blank or a separator right after the number, assume we really got a
@@ -1080,7 +1093,7 @@
 				doc = document_open_file(filename, FALSE, NULL, NULL);
 				if (doc != NULL)
 				{
-					ret = (line < 0) ? TRUE : navqueue_goto_line(old_doc, doc, line);
+					ret = navqueue_goto_line(old_doc, doc, line > 1 ? line : 1);
 					if (ret && focus_editor)
 						gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 				}
--- ./src/msgwindow.h.orig	2011-08-14 14:52:08.000000000 +0300
+++ ./src/msgwindow.h	2011-08-14 16:43:07.000000000 +0300
@@ -99,7 +99,7 @@
 gboolean msgwin_goto_compiler_file_line(gboolean focus_editor);
 
 void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
-									  gchar **filename, gint *line);
+									  gchar **filename, gint *line, gint *col);
 
 gboolean msgwin_goto_messages_file_line(gboolean focus_editor);
 
--- ./src/navqueue.c.orig	2011-01-21 20:15:58.000000000 +0200
+++ ./src/navqueue.c	2011-08-22 21:02:42.000000000 +0300
@@ -142,17 +142,37 @@
  *         position is set
  *  @param new_doc The document of the new position, must be valid.
  *  @param line the line number of the new position. It is counted with 1 as the first line, not 0.
+ *         line 0 works as line 1, line > number of lines goes to end of the document.
  *
  *  @return @c TRUE if the cursor has changed the position to @a line or @c FALSE otherwise.
  **/
 gboolean navqueue_goto_line(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line)
 {
+	return navqueue_goto_line_col(old_doc, new_doc, line, 0);
+}
+
+
+gboolean navqueue_goto_line_col(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line,
+		gint col)
+{
 	gint pos;
 
 	g_return_val_if_fail(new_doc != NULL, FALSE);
-	g_return_val_if_fail(line >= 1, FALSE);
+	g_return_val_if_fail(line >= 0, FALSE);
+
+	if (line > 0) /* some compilers, like pdflatex report errors on line 0 */
+		line--;   /* so only adjust the line number if it is greater than 0 */
+	pos = sci_get_position_from_line(new_doc->editor->sci, line);
+	if (pos < 0)  /* returned by get position if line # > number of lines + 1 */
+		pos = sci_get_length(new_doc->editor->sci); /* goto EOF like scintilla */
+	else if (col > 1)
+	{
+		gint eoln = sci_get_line_end_position(new_doc->editor->sci, line);
 
-	pos = sci_get_position_from_line(new_doc->editor->sci, line - 1);
+		pos += col - 1;
+		if (pos > eoln)
+			pos = eoln;
+	}
 
 	/* first add old file position */
 	if (old_doc != NULL && old_doc->file_name)
--- ./src/navqueue.h.orig	2011-01-21 20:15:58.000000000 +0200
+++ ./src/navqueue.h	2011-08-15 20:19:45.000000000 +0300
@@ -41,6 +41,9 @@
 
 gboolean navqueue_goto_line(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line);
 
+gboolean navqueue_goto_line_col(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line,
+		gint col);
+
 void navqueue_go_back(void);
 
 void navqueue_go_forward(void);
--- ./src/utils.c.orig	2011-08-14 14:52:08.000000000 +0300
+++ ./src/utils.c	2011-08-18 19:55:31.000000000 +0300
@@ -2161,3 +2161,20 @@
 	g_free(second);
 	return strv;
 }
+
+/* Converts @a s to a positive integer (including zero).
+ * If @a entire is TRUE, the entire string must be a number.
+ * On success, stores the value in the location pointed by i, and returns
+ * a pointer to the first unparsed character. On failure, returns NULL. */
+const gchar *utils_str_to_pos(const gchar *s, gint *i, gboolean entire)
+{
+	char *end;
+	unsigned long l = strtoul(s, &end, 10);
+
+	if (end > s && (*end == '\0' || !entire) && l <= G_MAXINT)
+	{
+		*i = l;
+		return end;
+	}
+	return NULL;
+}
--- ./src/utils.h.orig	2011-06-18 17:43:13.000000000 +0300
+++ ./src/utils.h	2011-08-18 19:51:10.000000000 +0300
@@ -269,4 +269,6 @@
 
 gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...) G_GNUC_NULL_TERMINATED;
 
+const gchar *utils_str_to_pos(const gchar *s, gint *i, gboolean entire);
+
 #endif
